@@ -3,7 +3,12 @@
 #include <pthread.h>
 #include "common.h"
 
-const char VERSION[] = "3.3.3";
+const char VERSION[] = "3.4.0";
+
+const char LAST_ROW[] = " [F2 Playlist|F3 Library|F4 Track|F5 Search|F6 Help]";
+
+const int MOUSE_DRAG = 32;
+const int MOUSE_CLICK = 0;
 
 double pauseSeconds = 0.0;
 double totalPauseSeconds = 0.0;
