@@ -110,6 +110,111 @@ bool isCooldownElapsed(int milliSeconds)
         return elapsedMilliseconds >= milliSeconds;
 }
 
+enum EventType getMouseLastRowEvent(int mouseXOnLastRow)
+{
+        enum EventType result = EVENT_NONE;
+
+        int viewClicked = 1;
+        for (int i = 0; i < mouseXOnLastRow; i++)
+        {
+                if (LAST_ROW[i] == '|')
+                {
+                        viewClicked++;
+                }
+        }
+        switch (viewClicked)
+        {
+        case 1:
+                result = EVENT_SHOWPLAYLIST;
+                break;
+        case 2:
+                result = EVENT_SHOWLIBRARY;
+                break;
+        case 3:
+                result = EVENT_SHOWTRACK;
+                break;
+        case 4:
+                result = EVENT_SHOWSEARCH;
+                break;
+        case 5:
+                result = EVENT_SHOWKEYBINDINGS;
+                break;
+        default:
+                result = EVENT_NONE;
+                break;
+        }
+        // Switch to library view if track view is clicked and no song is currently playing
+        if (result == EVENT_SHOWTRACK && getCurrentSongData() == NULL)
+        {
+                result = EVENT_SHOWLIBRARY;
+        }
+
+        return result;
+}
+
+bool mouseInputHandled(char *seq, int i, struct Event *event)
+{
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+        (void)term_w;
+
+        int mouseButton, mouseX, mouseY;
+        mouseButton = mouseX = mouseY = 0;
+        char *mouseTmp = strtok(seq + 3, ";");
+
+        if (mouseTmp != NULL)
+                mouseButton = getNumber(mouseTmp);
+        mouseTmp = strtok(NULL, ";");
+        if (mouseTmp != NULL)
+                mouseX = getNumber(mouseTmp);
+        mouseTmp = strtok(NULL, ";");
+        if (mouseTmp != NULL)
+                mouseY = getNumber(mouseTmp);
+
+        int indent = getIndent();
+
+        if (progressBarLength > 0)
+        {
+                draggedProgressBarCol = mouseX - progressBarCol;
+
+                float position = (float)draggedProgressBarCol / (float)progressBarLength;
+                double duration = getCurrentSongDuration();
+                draggedPositionSeconds = (float)duration * position;
+        }
+
+        // Clicked on last row
+        if (mouseY == lastRowRow && indent > 0 &&
+            mouseX - lastRowCol > 0 && mouseX - lastRowCol < (int)strlen(LAST_ROW) &&
+            mouseButton != MOUSE_DRAG)
+        {
+                event->type = getMouseLastRowEvent(mouseX - lastRowCol);
+                return true;
+        }
+        // Clicked on progress bar
+        else if ((mouseY == progressBarRow || draggingProgressBar) &&
+                 mouseX - progressBarCol >= 0 && mouseX - progressBarCol < progressBarLength &&
+                 appState.currentView == TRACK_VIEW)
+        {
+
+                if (mouseButton == MOUSE_DRAG || mouseButton == MOUSE_CLICK)
+                {
+                        draggingProgressBar = true;
+                        gint64 newPositionMicroSeconds = draggedPositionSeconds * G_USEC_PER_SEC;
+                        setPosition(newPositionMicroSeconds);
+                }
+                return true;
+        }
+
+        // Normal mouse event
+        if (strncmp(seq + 1, keyMappings[i].seq, strlen(keyMappings[i].seq)) == 0)
+        {
+                event->type = keyMappings[i].eventType;
+                return true;
+        }
+
+        return false;
+}
+
 struct Event processInput()
 {
         struct Event event;
@@ -172,7 +277,8 @@ struct Event processInput()
 
         if (!foundInput && cooldownElapsed)
         {
-                flushSeek();
+                if (!draggingProgressBar)
+                        flushSeek();
                 return event;
         }
 
@@ -206,6 +312,7 @@ struct Event processInput()
                 seq[0] = '\b'; // Treat as Backspace
         }
 
+        bool handledMouse = false;
         // Set event for pressed key
         for (int i = 0; i < NUM_KEY_MAPPINGS; i++)
         {
@@ -223,15 +330,20 @@ struct Event processInput()
                 }
 
                 // Received mouse input instead of keyboard input
-                if (keyMappings[i].seq[0] != '\0' && strnlen(seq, MAX_SEQ_LEN) > 3 && strncmp(seq, "\033[M", 3) == 0 &&
-                    ((strncmp(seq + 1, keyMappings[i].seq, 3) == 0) ||
-                     strncmp(seq, keyMappings[i].seq, 3) == 0))
+                if (keyMappings[i].seq[0] != '\0' && strncmp(seq, "\033[<", 3) == 0 && strnlen(seq, MAX_SEQ_LEN) > 4 && strchr(seq, 'M') != NULL &&
+                    mouseInputHandled(seq, i, &event))
                 {
-                        event.type = keyMappings[i].eventType;
+                        handledMouse = true;
                         break;
                 }
         }
 
+        if (!handledMouse)
+        {
+                // Stop dragging progress bar
+                draggingProgressBar = false;
+        }
+
         for (int i = 0; i < NUM_KEY_MAPPINGS; i++)
         {
                 if (strcmp(seq, "\033\n") == 0 && strcmp(keyMappings[i].seq, "^M") == 0) // ALT+ENTER
@@ -764,6 +876,9 @@ void handleInput(AppState *state)
         case EVENT_SEEKFORWARD:
                 seekForward(&(state->uiState));
                 break;
+        case EVENT_ADDTOFAVORITESPLAYLIST:
+                addToFavoritesPlaylist();
+                break;
         case EVENT_EXPORTPLAYLIST:
                 exportCurrentPlaylist(settings.path);
                 break;
@@ -1040,7 +1155,7 @@ void updatePlayerStatus(AppState *state)
 
                 if (isPlaybackDone())
                 {
-                        updateLastSongSwitchTime();
+                        resetStartTime();
                         prepareNextSong(state);
                         switchAudioImplementation();
                 }
@@ -1091,7 +1206,7 @@ static gboolean quitOnSignal(gpointer user_data)
 void initFirstPlay(Node *song, AppState *state)
 {
         updateLastInputTime();
-        updateLastSongSwitchTime();
+        resetStartTime();
 
         userData.currentSongData = NULL;
         userData.songdataA = NULL;
@@ -1191,11 +1306,14 @@ void cleanupOnExit()
         restoreTerminalMode();
         enableInputBuffering();
         setConfig(&settings, &(appState.uiSettings));
+        saveFavoritesPlaylist(settings.path);
         saveLastUsedPlaylist();
         deleteCache(appState.tmpCache);
         freeMainDirectoryTree(&appState);
         deletePlaylist(&playlist);
         deletePlaylist(originalPlaylist);
+        deletePlaylist(favoritesPlaylist);
+        free(favoritesPlaylist);
         free(originalPlaylist);
         setDefaultTextColor();
         pthread_mutex_destroy(&(loadingdata.mutex));
@@ -1223,6 +1341,9 @@ void cleanupOnExit()
         if (appState.uiSettings.mouseEnabled)
                 disableTerminalMouseButtons();
 
+        if (appState.uiSettings.trackTitleAsWindowTitle)
+                restoreTerminalWindowTitle();
+
         if (noMusicFound)
         {
                 printf("No Music found.\n");
@@ -1357,6 +1478,21 @@ void initDefaultState(AppState *state)
         run(state, false);
 }
 
+void playFavoritesPlaylist(AppState *state)
+{
+        if (favoritesPlaylist->count == 0)
+        {
+                printf("Couldn't find any songs in the special playlist. Add a song by pressing '.' while it's playing. \n");
+                exit(0);
+        }
+
+        init(state);
+        deepCopyPlayListOntoList(favoritesPlaylist, &playlist);
+        shufflePlaylist(&playlist);
+        markListAsEnqueued(library, &playlist);
+        run(state, true);
+}
+
 void playAll(AppState *state)
 {
         init(state);
@@ -1366,8 +1502,7 @@ void playAll(AppState *state)
         {
                 exit(0);
         }
-        if (!state->uiSettings.saveRepeatShuffleSettings)
-                shufflePlaylist(&playlist);
+        shufflePlaylist(&playlist);
         markListAsEnqueued(library, &playlist);
         run(state, true);
 }
@@ -1648,8 +1783,15 @@ void setMusicPath()
 void enableMouse(UISettings *ui)
 {
         if (ui->mouseEnabled)
-        {
                 enableTerminalMouseButtons();
+}
+
+void setTrackTitleAsWindowTitle(UISettings *ui)
+{
+        if (ui->trackTitleAsWindowTitle)
+        {
+                saveTerminalWindowTitle();
+                setTerminalWindowTitle("kew");
         }
 }
 
@@ -1685,6 +1827,7 @@ void initState(AppState *state)
         state->uiSettings.saveRepeatShuffleSettings = 1;
         state->uiSettings.repeatState = 0;
         state->uiSettings.shuffleEnabled = 0;
+        state->uiSettings.trackTitleAsWindowTitle = 1;
         state->uiState.numDirectoryTreeEntries = 0;
         state->uiState.numProgressBars = 35;
         state->uiState.chosenNodeId = 0;
@@ -1706,6 +1849,7 @@ void initializeStateAndSettings(AppState *appState, AppSettings *settings)
         userData.replayGainCheckFirst = appState->uiSettings.replayGainCheckFirst;
         mapSettingsToKeys(settings, &(appState->uiSettings), keyMappings);
         enableMouse(&(appState->uiSettings));
+        setTrackTitleAsWindowTitle(&(appState->uiSettings));
 }
 
 int main(int argc, char *argv[])
@@ -1743,6 +1887,7 @@ int main(int argc, char *argv[])
         }
 
         handleOptions(&argc, argv, ui);
+        loadFavoritesPlaylist(settings.path);
 
         if (argc == 1)
         {
@@ -1756,6 +1901,10 @@ int main(int argc, char *argv[])
         {
                 playAllAlbums(&appState);
         }
+        else if (argc == 2 && strcmp(argv[1], ".") == 0 && favoritesPlaylist->count != 0)
+        {
+                playFavoritesPlaylist(&appState);
+        }
         else if (argc >= 2)
         {
                 init(&appState);
