@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include "imgfunc.h"
@@ -31,7 +32,7 @@ player_ui.c
 #endif
 
 #ifdef __APPLE__
-const int ABSOLUTE_MIN_WIDTH = 65;
+const int ABSOLUTE_MIN_WIDTH = 80;
 #else
 const int ABSOLUTE_MIN_WIDTH = 65;
 #endif
@@ -59,6 +60,15 @@ int libSongIter = 0;
 int libTopLevelSongIter = 0;
 int previousChosenLibRow = 0;
 int libCurrentDirSongCount = 0;
+int lastRowRow = 0;
+int lastRowCol = 0;
+int progressBarRow = 0;
+int progressBarCol = 0;
+int progressBarLength = 0;
+int draggedProgressBarCol = 0;
+double draggedPositionSeconds = 0.0;
+bool draggingProgressBar = false;
+int miniVisualizerRow = 0;
 
 PixelData lastRowColor = {120, 120, 120};
 
@@ -128,6 +138,7 @@ void printHelp()
                "          kew dir <album name> (Sometimes it's necessary to specify it's a directory you want)\n"
                "          kew song <song name> \n"
                "          kew list <m3u list name> \n"
+               "          kew . (plays kew favorites.m3u file)\n"
                "          kew shuffle <dir name> (random and rand works too)\n"
                "          kew artistA:artistB (plays artistA and artistB shuffled)\n"
                "\n"
@@ -147,6 +158,7 @@ void printHelp()
                " F4 to show/hide track view.\n"
                " F5 to show/hide search view.\n"
                " F6 to show/hide show/hide key bindings view.\n"
+               " You can also use the mouse to switch views.\n"
                " u to update the library.\n"
                " v to toggle the spectrum visualizer.\n"
                " i to switch between using your regular color scheme or colors derived from the track cover.\n"
@@ -163,6 +175,7 @@ void printHelp()
                " gg go to first song.\n"
                " number + G or Enter to go to specific song number in the playlist.\n"
                " G to go to last song.\n"
+               " . to add currently playing song to \"kew favorites.m3u\" (run with \"kew .\").\n"
                " Esc or q to quit.\n"
                "\n");
 }
@@ -215,7 +228,6 @@ int printLogo(SongData *songData, UISettings *ui)
                 strncpy(prettyTitle, songData->metadata->title, titleLength);
                 prettyTitle[titleLength] = '\0';
 
-                removeUnneededChars(prettyTitle, titleLength);
                 trim(prettyTitle, titleLength);
 
                 if (ui->hideLogo && songData->metadata->artist[0] != '\0')
@@ -260,7 +272,7 @@ int getYear(const char *dateString)
         return year;
 }
 
-void printCover(SongData *songdata, UISettings *ui)
+void printCoverCentered(SongData *songdata, UISettings *ui)
 {
         clearScreen();
 
@@ -272,7 +284,7 @@ void printCover(SongData *songdata, UISettings *ui)
                 }
                 else
                 {
-                        printInAscii(songdata->coverArtPath, preferredHeight);
+                        printInAsciiCentered(songdata->coverArtPath, preferredHeight);
                 }
         }
         else
@@ -286,7 +298,28 @@ void printCover(SongData *songdata, UISettings *ui)
         printf("\n\n");
 }
 
-void printTitleWithDelay(const char *text, int delay, int maxWidth)
+void printCover(int height, SongData *songdata, UISettings *ui)
+{
+        int row = 2;
+        int col = 2;
+        int imgHeight = height - 2;
+
+        clearScreen();
+
+        if (songdata != NULL && songdata->cover != NULL && ui->coverEnabled)
+        {
+                if (!ui->coverAnsi)
+                {
+                        printSquareBitmap(row, col, songdata->cover, songdata->coverWidth, songdata->coverHeight, imgHeight);
+                }
+                else
+                {
+                        printInAscii(col, songdata->coverArtPath, imgHeight);
+                }
+        }
+}
+
+void printTitleWithDelay(int row, int col, const char *text, int delay, int maxWidth)
 {
         int max = strnlen(text, maxWidth);
 
@@ -295,8 +328,9 @@ void printTitleWithDelay(const char *text, int delay, int maxWidth)
 
         for (int i = 0; i <= max && delay; i++)
         {
-                printf("\r ");
-                printBlankSpaces(indent);
+                printf("\033[%d;%dH", row, col);
+                printf("\033[K");
+
                 for (int j = 0; j < i; j++)
                 {
                         printf("%c", text[j]);
@@ -309,90 +343,73 @@ void printTitleWithDelay(const char *text, int delay, int maxWidth)
         if (delay)
                 c_sleep(delay * 20);
 
-        printf("\r");
+        printf("\033[%d;%dH", row, col);
         printf("\033[K");
-        printBlankSpaces(indent);
-        printf("\033[1K %.*s", maxWidth, text);
+        printf("%.*s", maxWidth, text);
         printf("\n");
         fflush(stdout);
 }
 
-void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
+void printBasicMetadata(int row, int col, int maxWidth, TagSettings const *metadata, UISettings *ui)
 {
-        int term_w, term_h;
-        getTermSize(&term_w, &term_h);
-        int maxWidth = textWidth; // term_w - 3 - (indent * 2);
-        printf("\n");
+        if (ui->useConfigColors)
+                setDefaultTextColor();
+        else
+                setTextColorRGB(ui->color.r, ui->color.g, ui->color.b);
 
         if (strnlen(metadata->artist, METADATA_MAX_LENGTH) > 0)
         {
-                printBlankSpaces(indent);
-                printf(" %.*s\n", maxWidth, metadata->artist);
-        }
-        else
-        {
-                printf("\n");
+                printf("\033[%d;%dH", row + 1, col);
+                printf(" %.*s", maxWidth, metadata->artist);
         }
+
         if (strnlen(metadata->album, METADATA_MAX_LENGTH) > 0)
         {
-                printBlankSpaces(indent);
-                printf(" %.*s\n", maxWidth, metadata->album);
-        }
-        else
-        {
-                printf("\n");
+                printf("\033[%d;%dH", row + 2, col);
+                printf(" %.*s", maxWidth, metadata->album);
         }
+
         if (strnlen(metadata->date, METADATA_MAX_LENGTH) > 0)
         {
-                printBlankSpaces(indent);
+                printf("\033[%d;%dH", row + 3, col);
                 int year = getYear(metadata->date);
                 if (year == -1)
-                        printf(" %s\n", metadata->date);
+                        printf(" %s", metadata->date);
                 else
-                        printf(" %d\n", year);
+                        printf(" %d", year);
+        }
+
+        PixelData pixel = increaseLuminosity(ui->color, 20);
+
+        if (ui->useConfigColors)
+        {
+                setDefaultTextColor();
+        }
+        else if (pixel.r == 255 && pixel.g == 255 && pixel.b == 255)
+        {
+                PixelData gray;
+                gray.r = defaultColor;
+                gray.g = defaultColor;
+                gray.b = defaultColor;
+                printf("\033[1;38;2;%03u;%03u;%03um", gray.r, gray.g, gray.b);
         }
         else
         {
-                printf("\n");
+                printf("\033[1;38;2;%03u;%03u;%03um", pixel.r, pixel.g, pixel.b);
         }
-        cursorJump(4);
+
         if (strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
         {
-                PixelData pixel = increaseLuminosity(ui->color, 20);
-
-                if (ui->useConfigColors)
-                {
-                        setDefaultTextColor();
-                }
-                else if (pixel.r == 255 && pixel.g == 255 && pixel.b == 255)
-                {
-                        PixelData gray;
-                        gray.r = defaultColor;
-                        gray.g = defaultColor;
-                        gray.b = defaultColor;
-                        printf("\033[1;38;2;%03u;%03u;%03um", gray.r, gray.g, gray.b);
-                }
-                else
-                {
-                        printf("\033[1;38;2;%03u;%03u;%03um", pixel.r, pixel.g, pixel.b);
-                }
-
                 // Clean up title before printing
                 int titleLength = strnlen(metadata->title, maxWidth);
                 char prettyTitle[titleLength + 1];
 
                 c_strcpy(prettyTitle, metadata->title, titleLength + 1);
 
-                removeUnneededChars(prettyTitle, titleLength);
                 trim(prettyTitle, titleLength);
 
-                printTitleWithDelay(prettyTitle, ui->titleDelay, maxWidth);
+                printTitleWithDelay(row, col + 1, prettyTitle, ui->titleDelay, maxWidth);
         }
-        else
-        {
-                printf("\n");
-        }
-        cursorJumpDown(3);
 }
 
 int calcElapsedBars(double elapsedSeconds, double duration, int numProgressBars)
@@ -412,9 +429,6 @@ void printProgress(double elapsed_seconds, double total_seconds, ma_uint32 sampl
         if (term_w < progressWidth)
                 return;
 
-        // Save the current cursor position
-        printf("\033[s");
-
         int elapsed_hours = (int)(elapsed_seconds / 3600);
         int elapsed_minutes = (int)(((int)elapsed_seconds / 60) % 60);
         int elapsed_seconds_remainder = (int)elapsed_seconds % 60;
@@ -426,11 +440,6 @@ void printProgress(double elapsed_seconds, double total_seconds, ma_uint32 sampl
         int progress_percentage = (int)((elapsed_seconds / total_seconds) * 100);
         int vol = getCurrentVolume();
 
-        // Clear the current line
-        printf("\r\033[K");
-
-        printBlankSpaces(indent);
-
         if (total_seconds >= 3600)
         {
                 // Song is more than 1 hour long: use full HH:MM:SS format
@@ -458,7 +467,7 @@ void printProgress(double elapsed_seconds, double total_seconds, ma_uint32 sampl
         if (term_w > progressWidth + 10)
         {
                 if (rate == (int)rate)
-                        printf(" %dkHz", (int)rate); // No decimals
+                        printf(" %dkHz", (int)rate);
                 else
                         printf(" %.1fkHz", rate);
         }
@@ -468,27 +477,9 @@ void printProgress(double elapsed_seconds, double total_seconds, ma_uint32 sampl
                 if (avgBitRate > 0)
                         printf(" %dkb/s ", avgBitRate);
         }
-
-        // Restore the cursor position
-        printf("\r");
 }
 
-void printMetadata(TagSettings const *metadata, UISettings *ui)
-{
-        if (appState.currentView == LIBRARY_VIEW || appState.currentView == PLAYLIST_VIEW || appState.currentView == SEARCH_VIEW)
-                return;
-
-        if (ui->titleDelay)
-                c_sleep(100);
-
-        if (ui->useConfigColors)
-                setDefaultTextColor();
-        else
-                setTextColorRGB(ui->color.r, ui->color.g, ui->color.b);
-        printBasicMetadata(metadata, ui);
-}
-
-void printTime(double elapsedSeconds, ma_uint32 sampleRate, int avgBitRate, AppState *state)
+void printTime(int row, int col, double elapsedSeconds, ma_uint32 sampleRate, int avgBitRate, AppState *state)
 {
         if (state->uiSettings.useConfigColors)
                 setDefaultTextColor();
@@ -496,11 +487,13 @@ void printTime(double elapsedSeconds, ma_uint32 sampleRate, int avgBitRate, AppS
                 setTextColorRGB(state->uiSettings.color.r, state->uiSettings.color.g, state->uiSettings.color.b);
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
-        printBlankSpaces(indent);
+        printf("\033[%d;%dH", row, col);
         if (term_h > minHeight)
         {
                 double duration = getCurrentSongDuration();
-                printProgress(elapsedSeconds, duration, sampleRate, avgBitRate);
+                double elapsed = elapsedSeconds;
+
+                printProgress(elapsed, duration, sampleRate, avgBitRate);
         }
 }
 
@@ -542,12 +535,19 @@ void calcIndent(SongData *songdata)
         }
 }
 
-void printGlimmeringText(char *text, int textLength, char *nerdFontText, PixelData color, int indent)
+int getIndent()
+{
+        return indent;
+}
+
+void printGlimmeringText(int row, int col, char *text, int textLength, char *nerdFontText, PixelData color)
 {
         int brightIndex = 0;
         PixelData vbright = increaseLuminosity(color, 120);
         PixelData bright = increaseLuminosity(color, 60);
 
+        printf("\033[%d;%dH", row, col);
+
         while (brightIndex < textLength)
         {
                 for (int i = 0; i < textLength; i++)
@@ -576,44 +576,43 @@ void printGlimmeringText(char *text, int textLength, char *nerdFontText, PixelDa
                 c_usleep(50);
 
                 brightIndex++;
-                printf("\r");
-                printBlankSpaces(indent);
+                printf("\033[%d;%dH", row, col);
         }
 }
 
-void printErrorRow(void)
+void printErrorRow(int row, int col)
 {
-        int indent = calcIndentNormal();
-
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
 
-#ifndef __APPLE__
-        // Move to next to lastRow
-        printf("\033[%d;1H", term_h - 1);
-#endif
-
-        if (term_w < ABSOLUTE_MIN_WIDTH)
-        {
-                printf("\n");
-                return;
-        }
+        printf("\033[%d;%dH", row, col);
 
         if (!hasPrintedError && hasErrorMessage())
         {
                 setTextColorRGB(lastRowColor.r, lastRowColor.g, lastRowColor.b);
-                printBlankSpaces(indent);
-                printf(" %s\n", getErrorMessage());
+                printf(" %s", getErrorMessage());
                 hasPrintedError = true;
                 fflush(stdout);
         }
         else
         {
-                printf("\n");
+                printf("\033[K"); // Clear the rest of the line
         }
 }
 
-void printLastRow(UISettings *ui)
+void formatWithShiftPlus(char *dest, size_t size, const char *src)
+{
+        if (isupper((unsigned char)src[0]))
+        {
+                snprintf(dest, size, "Shift+%s", src);
+        }
+        else
+        {
+                snprintf(dest, size, "%s", src);
+        }
+}
+
+void printLastRow(int row, int col, UISettings *ui, AppSettings *settings)
 {
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
@@ -621,22 +620,34 @@ void printLastRow(UISettings *ui)
         if (preferredWidth < 0 || preferredHeight < 0) // mini view
                 return;
 
-#ifndef __APPLE__
-        // Move to lastRow
-        printf("\033[%d;1H", term_h);
-#endif
+        lastRowRow = row;
+        lastRowCol = col;
+
+        printf("\033[%d;%dH", row, col);
+
         setTextColorRGB(lastRowColor.r, lastRowColor.g, lastRowColor.b);
 
+        char text[100];
 #ifdef __APPLE__
-        char text[100] = " Sh+Z List|Sh+X Lib|Sh+C Track|Sh+V Search|Sh+B Help";
+        char playlist[8], library[8], track[8], search[8], help[8];
+
+        // Assume settings->showPlaylistAlt etc. are defined properly
+        formatWithShiftPlus(playlist, sizeof(playlist), settings->showPlaylistAlt);
+        formatWithShiftPlus(library, sizeof(library), settings->showLibraryAlt);
+        formatWithShiftPlus(track, sizeof(track), settings->showTrackAlt);
+        formatWithShiftPlus(search, sizeof(search), settings->showSearchAlt);
+        formatWithShiftPlus(help, sizeof(help), settings->showKeysAlt);
+
+        snprintf(text, sizeof(text), "%s Playlist|%s Library|%s Track|%s Search|%s Help",
+                 playlist, library, track, search, help);
+
 #else
-        char text[100] = " [F2 Playlist|F3 Library|F4 Track|F5 Search|F6 Help]";
+        (void)settings;
+        strcpy(text, LAST_ROW);
 #endif
 
         char nerdFontText[100] = "";
 
-        printf("\r");
-
         size_t maxLength = sizeof(nerdFontText);
 
         size_t currentLength = strnlen(nerdFontText, maxLength);
@@ -647,19 +658,19 @@ void printLastRow(UISettings *ui)
                 {
                         char pauseText[] = " ⏸";
                         snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
-                        currentLength += strnlen(pauseText, maxLength - currentLength);
+                        currentLength += strlen(pauseText);
                 }
                 else if (isStopped())
                 {
                         char pauseText[] = " ■";
                         snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
-                        currentLength += strnlen(pauseText, maxLength - currentLength);
+                        currentLength += strlen(pauseText);
                 }
                 else
                 {
                         char pauseText[] = " ▶";
                         snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
-                        currentLength += strnlen(pauseText, maxLength - currentLength);
+                        currentLength += strlen(pauseText);
                 }
         }
 
@@ -667,34 +678,34 @@ void printLastRow(UISettings *ui)
         {
                 char repeatText[] = " \u27f3";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", repeatText);
-                currentLength += strnlen(repeatText, maxLength - currentLength);
+                currentLength += strlen(repeatText);
         }
         else if (isRepeatListEnabled())
         {
                 char repeatText[] = " \u27f3L";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", repeatText);
-                currentLength += strnlen(repeatText, maxLength - currentLength);
+                currentLength += strlen(repeatText);
         }
 
         if (isShuffleEnabled())
         {
                 char shuffleText[] = " \uf074";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", shuffleText);
-                currentLength += strnlen(shuffleText, maxLength - currentLength);
+                currentLength += strlen(shuffleText);
         }
 
         if (fastForwarding)
         {
                 char forwardText[] = " \uf04e";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", forwardText);
-                currentLength += strnlen(forwardText, maxLength - currentLength);
+                currentLength += strlen(forwardText);
         }
 
         if (rewinding)
         {
                 char rewindText[] = " \uf04a";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", rewindText);
-                currentLength += strnlen(rewindText, maxLength - currentLength);
+                currentLength += strlen(rewindText);
         }
 
         printf("\033[K"); // Clear the line
@@ -703,26 +714,32 @@ void printLastRow(UISettings *ui)
         {
                 if (term_w > (int)currentLength + indent)
                 {
-                        printBlankSpaces(indent);
                         printf("%s", nerdFontText); // Print just the shuffle and replay settings
                 }
                 return;
         }
 
-        int indent = calcIndentNormal();
         int textLength = strnlen(text, 100);
         int randomNumber = getRandomNumber(1, 808);
 
         if (randomNumber == 808 && !ui->hideGlimmeringText)
-                printGlimmeringText(text, textLength, nerdFontText, lastRowColor, indent);
+                printGlimmeringText(row, col, text, textLength, nerdFontText, lastRowColor);
         else
         {
-                printBlankSpaces(indent);
                 printf("%s", text);
                 printf("%s", nerdFontText);
         }
 }
 
+void calcAndPrintLastRowAndErrorRow(UISettings *ui, AppSettings *settings)
+{
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+
+        printErrorRow(term_h - 1, indent);
+        printLastRow(term_h, indent, ui, settings);
+}
+
 int printAbout(SongData *songdata, UISettings *ui)
 {
         clearScreen();
@@ -771,6 +788,8 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         printBlankSpaces(indent);
         printf(" - Shift+s to stop.\n");
         printBlankSpaces(indent);
+        printf(" - You can also use the mouse to switch views.\n");
+        printBlankSpaces(indent);
         printf(" - %s toggle color derived from album or from profile.\n", settings->toggleColorsDerivedFrom);
         printBlankSpaces(indent);
         printf(" - %s to update the library.\n", settings->updateLibrary);
@@ -791,6 +810,8 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         printBlankSpaces(indent);
         printf("   in an .m3u file named after the first song in the playlist.\n");
         printBlankSpaces(indent);
+        printf(" - %s to add current song to kew favorites.m3u (run with \"kew .\").\n", settings->addToFavoritesPlaylist);
+        printBlankSpaces(indent);
         printf(" - Esc or %s to quit.\n\n", settings->quit);
         printBlankSpaces(indent);
         printf(" Project URL: https://github.com/ravachol/kew\n");
@@ -806,8 +827,8 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
                 numPrintedRows++;
         }
 
-        printErrorRow();
-        printLastRow(ui);
+        calcAndPrintLastRowAndErrorRow(ui, settings);
+
         numPrintedRows++;
 
         return numPrintedRows;
@@ -1002,7 +1023,7 @@ int printLogoAndAdjustments(SongData *songData, int termWidth, UISettings *ui, i
         return aboutRows;
 }
 
-void showSearch(SongData *songData, int *chosenRow, UISettings *ui)
+void showSearch(SongData *songData, int *chosenRow, UISettings *ui, AppSettings *settings)
 {
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
@@ -1022,11 +1043,10 @@ void showSearch(SongData *songData, int *chosenRow, UISettings *ui)
 
         displaySearch(maxSearchListSize, indent, chosenRow, startSearchIter, ui);
 
-        printErrorRow();
-        printLastRow(ui);
+        calcAndPrintLastRowAndErrorRow(ui, settings);
 }
 
-void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chosenNodeId, AppState *state)
+void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chosenNodeId, AppState *state, AppSettings *settings)
 {
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
@@ -1058,8 +1078,7 @@ void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chos
 
         displayPlaylist(list, maxListSize, indent, chosenSong, chosenNodeId, state->uiState.resetPlaylistDisplay, state);
 
-        printErrorRow();
-        printLastRow(&(state->uiSettings));
+        calcAndPrintLastRowAndErrorRow(&(state->uiSettings), settings);
 }
 
 void resetSearchResult(void)
@@ -1067,13 +1086,17 @@ void resetSearchResult(void)
         chosenSearchResultRow = 0;
 }
 
-void printElapsedChars(AppSettings *settings, UISettings *ui, int elapsedBars, int numProgressBars)
+void printProgressBar(int row, int col, AppSettings *settings, UISettings *ui, int elapsedBars, int numProgressBars)
 {
         PixelData color = ui->color;
         bool useConfigColors = ui->useConfigColors;
 
-        printBlankSpaces(indent);
-        printf(" ");
+        progressBarRow = row;
+        progressBarCol = col + 1;
+        progressBarLength = numProgressBars;
+
+        printf("\033[%d;%dH", row, col + 1);
+
         for (int i = 0; i < numProgressBars; i++)
         {
                 if (i > elapsedBars)
@@ -1122,62 +1145,22 @@ void printElapsedChars(AppSettings *settings, UISettings *ui, int elapsedBars, i
         }
 }
 
-void printVisualizer(AppSettings *settings, double elapsedSeconds, AppState *state)
+void printVisualizer(int row, int col, int visualizerWidth, AppSettings *settings, double elapsedSeconds, AppState *state)
 {
         UISettings *ui = &(state->uiSettings);
         UIState *uis = &(state->uiState);
-        int term_w, term_h;
-        getTermSize(&term_w, &term_h);
 
         if (ui->visualizerEnabled)
         {
-                printf("\n");
-
-                int visualizerWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
-                visualizerWidth = (visualizerWidth < textWidth && textWidth < term_w - 2) ? textWidth : visualizerWidth;
-                visualizerWidth = (visualizerWidth > term_w - 2) ? term_w - 2 : visualizerWidth;
-                visualizerWidth -= 1;
                 uis->numProgressBars = (int)visualizerWidth / 2;
                 double duration = getCurrentSongDuration();
 
-#ifndef __APPLE__
-                saveCursorPosition();
-#endif
-                drawSpectrumVisualizer(state, indent);
+                drawSpectrumVisualizer(row, col, state);
 
-                printElapsedChars(settings, ui, calcElapsedBars(elapsedSeconds, duration, uis->numProgressBars * 2), uis->numProgressBars * 2 - 1);
-                printErrorRow();
-                printLastRow(&(state->uiSettings));
-#ifndef __APPLE__
-                restoreCursorPosition();
-                cursorJump(1);
-#else
-                int jumpAmount = ui->visualizerHeight + 2;
-                cursorJump(jumpAmount);
-#endif
-        }
-        else if (!ui->visualizerEnabled)
-        {
-                if (term_w >= ABSOLUTE_MIN_WIDTH)
-                {
-#ifdef __APPLE__
-                        printf("\n");
-                        printErrorRow();
-                        saveCursorPosition();
-                        printLastRow(ui);
-                        restoreCursorPosition();
-                        cursorJump(2);
-#else
-                        saveCursorPosition();
-                        printErrorRow();
-                        restoreCursorPosition();
-                        printf("\n");
-                        saveCursorPosition();
-                        printLastRow(ui);
-                        restoreCursorPosition();
-                        cursorJump(1);
-#endif
-                }
+                int height = state->uiSettings.visualizerHeight;
+                int elapsedBars = calcElapsedBars(elapsedSeconds, duration, visualizerWidth);
+
+                printProgressBar(row + height - 1, col, settings, ui, elapsedBars, visualizerWidth - 1);
         }
 }
 
@@ -1449,7 +1432,7 @@ char *getLibraryFilePath(void)
         return getFilePath(LIBRARY_FILE);
 }
 
-void showLibrary(SongData *songData, AppState *state)
+void showLibrary(SongData *songData, AppState *state, AppSettings *settings)
 {
         // For scrolling names, update every nth time
         if (getIsLongName() && isSameNameAsLastTime && updateCounter % scrollingInterval != 0)
@@ -1549,25 +1532,41 @@ void showLibrary(SongData *songData, AppState *state)
                 printf("\n");
         }
 
-        printErrorRow();
-        printLastRow(ui);
+        calcAndPrintLastRowAndErrorRow(ui, settings);
 
         if (!foundChosen && refresh)
         {
                 printf("\033[1;1H");
                 clearScreen();
-                showLibrary(songData, state);
+                showLibrary(songData, state, settings);
         }
 }
 
+int calcVisualizerWidth()
+{
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+        int visualizerWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
+        visualizerWidth = (visualizerWidth < textWidth && textWidth < term_w - 2) ? textWidth : visualizerWidth;
+        visualizerWidth = (visualizerWidth > term_w - 2) ? term_w - 2 : visualizerWidth;
+        visualizerWidth -= 1;
+
+        return visualizerWidth;
+}
+
 void showTrackViewMini(AppSettings *settings, SongData *songdata, AppState *state, double elapsedSeconds)
 {
         TagSettings *metadata = NULL;
         int avgBitRate = 0;
 
+        int col = indent;
+
         if (refresh)
         {
+                miniVisualizerRow = 1;
+
                 printf("\n");
+                miniVisualizerRow++;
 
                 clearScreen();
 
@@ -1586,6 +1585,7 @@ void showTrackViewMini(AppSettings *settings, SongData *songdata, AppState *stat
                                 printf("\n");
                                 printBlankSpaces(indent);
                                 printf(" kew version: %s", VERSION);
+                                miniVisualizerRow++;
                         }
                         return;
                 }
@@ -1610,16 +1610,16 @@ void showTrackViewMini(AppSettings *settings, SongData *songdata, AppState *stat
                         char combined[text_len + 1];
                         snprintf(combined, sizeof(combined), "%s - %s", metadata->artist, metadata->title);
 
-                        printf(" %.*s\n", text_len, combined);
+                        printf("%.*s\n", text_len, combined);
+                        miniVisualizerRow++;
                 }
                 else if (strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
                 {
                         printBlankSpaces(indent);
-                        printf(" %.*s\n", textWidth, metadata->title);
+                        printf("%.*s\n", textWidth, metadata->title);
+                        miniVisualizerRow++;
                 }
 
-                if (!songdata && metadata)
-                        free(metadata);
                 refresh = false;
         }
 
@@ -1635,17 +1635,34 @@ void showTrackViewMini(AppSettings *settings, SongData *songdata, AppState *stat
                 ma_format format;
                 avgBitRate = songdata->avgBitRate;
                 getCurrentFormatAndSampleRate(&format, &sampleRate);
-                printTime(elapsedSeconds, sampleRate, avgBitRate, state);
+                printTime(miniVisualizerRow - 1, col, elapsedSeconds, sampleRate, avgBitRate, state);
         }
         if (doPrintVis)
-                printVisualizer(settings, elapsedSeconds, state);
+        {
+                int visualizerWidth = calcVisualizerWidth();
+                printVisualizer(miniVisualizerRow, col, visualizerWidth, settings, elapsedSeconds, state);
+        }
 }
 
-void showTrackView(AppSettings *settings, SongData *songdata, AppState *state, double elapsedSeconds)
+void showTrackViewLandscape(int height, int width, float aspectRatio, AppSettings *settings, SongData *songdata, AppState *state, double elapsedSeconds)
 {
         TagSettings *metadata = NULL;
         int avgBitRate = 0;
 
+        int metadataHeight = 4;
+        int timeHeight = 1;
+
+        int col = height * aspectRatio;
+
+        if (!state->uiSettings.coverEnabled)
+                col = 1;
+
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+        int visualizerWidth = term_w - col;
+
+        int row = height - metadataHeight - timeHeight - state->uiSettings.visualizerHeight - 3;
+
         if (refresh)
         {
                 if (songdata)
@@ -1653,14 +1670,10 @@ void showTrackView(AppSettings *settings, SongData *songdata, AppState *state, d
                         metadata = songdata->metadata;
                 }
 
-                clearScreen();
-                printf("\n");
-                printCover(songdata, &(state->uiSettings));
+                printCover(height, songdata, &(state->uiSettings));
+                if (height > metadataHeight)
+                        printBasicMetadata(row, col, visualizerWidth-1, metadata, &(state->uiSettings));
 
-                printMetadata(metadata, &(state->uiSettings));
-
-                if (!songdata && metadata)
-                        free(metadata);
                 refresh = false;
         }
         if (songdata)
@@ -1669,9 +1682,75 @@ void showTrackView(AppSettings *settings, SongData *songdata, AppState *state, d
                 ma_format format;
                 avgBitRate = songdata->avgBitRate;
                 getCurrentFormatAndSampleRate(&format, &sampleRate);
-                printTime(elapsedSeconds, sampleRate, avgBitRate, state);
+                if (height > metadataHeight + timeHeight)
+                        printTime(row + 4, col, elapsedSeconds, sampleRate, avgBitRate, state);
+        }
+
+        if (row > 0)
+                printVisualizer(row + metadataHeight + 2, col, visualizerWidth, settings, elapsedSeconds, state);
+
+        if (width - col > ABSOLUTE_MIN_WIDTH)
+        {
+                printErrorRow(row + metadataHeight + 2 + state->uiSettings.visualizerHeight, col);
+                printLastRow(row + metadataHeight + 2 + state->uiSettings.visualizerHeight + 1, col, &(state->uiSettings), settings);
+        }
+}
+
+void showTrackViewPortrait(int height, AppSettings *settings, SongData *songdata, AppState *state, double elapsedSeconds)
+{
+        TagSettings *metadata = NULL;
+        int avgBitRate = 0;
+
+        int metadataHeight = 4;
+
+        int row = height + 3;
+        int col = indent;
+
+        int visualizerWidth = calcVisualizerWidth();
+
+        if (refresh)
+        {
+                if (songdata)
+                {
+                        metadata = songdata->metadata;
+                }
+
+                printCoverCentered(songdata, &(state->uiSettings));
+                printBasicMetadata(row, col, visualizerWidth-1, metadata, &(state->uiSettings));
+
+                refresh = false;
+        }
+        if (songdata)
+        {
+                ma_uint32 sampleRate;
+                ma_format format;
+                avgBitRate = songdata->avgBitRate;
+                getCurrentFormatAndSampleRate(&format, &sampleRate);
+                printTime(row + metadataHeight, col, elapsedSeconds, sampleRate, avgBitRate, state);
+        }
+
+        printVisualizer(row + metadataHeight + 2, col, visualizerWidth, settings, elapsedSeconds, state);
+
+        calcAndPrintLastRowAndErrorRow(&(state)->uiSettings, settings);
+}
+
+void showTrackView(int width, int height, AppSettings *settings, SongData *songdata, AppState *state, double elapsedSeconds)
+{
+        float aspect = getAspectRatio();
+
+        if (aspect == 0.0f)
+                aspect = 1.0f;
+
+        int correctedWidth = width / aspect;
+
+        if (correctedWidth > height * 2)
+        {
+                showTrackViewLandscape(height, width, aspect, settings, songdata, state, elapsedSeconds);
+        }
+        else
+        {
+                showTrackViewPortrait(preferredHeight, settings, songdata, state, elapsedSeconds);
         }
-        printVisualizer(settings, elapsedSeconds, state);
 }
 
 int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings, AppState *state)
@@ -1690,7 +1769,6 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         if (refresh)
         {
                 hideCursor();
-
                 setColor(ui);
 
                 if (songdata != NULL && songdata->metadata != NULL && !songdata->hasErrors && (songdata->hasErrors < 1))
@@ -1698,6 +1776,9 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
                         ui->color.r = songdata->red;
                         ui->color.g = songdata->green;
                         ui->color.b = songdata->blue;
+
+                        if (ui->trackTitleAsWindowTitle)
+                                setTerminalWindowTitle(songdata->metadata->title);
                 }
                 else
                 {
@@ -1709,6 +1790,9 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
                         ui->color.r = defaultColor;
                         ui->color.g = defaultColor;
                         ui->color.b = defaultColor;
+
+                        if (ui->trackTitleAsWindowTitle)
+                                setTerminalWindowTitle("kew");
                 }
 
                 calcPreferredSize(ui);
@@ -1740,25 +1824,25 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         }
         else if (state->currentView == PLAYLIST_VIEW && refresh)
         {
-                showPlaylist(songdata, originalPlaylist, &chosenRow, &(uis->chosenNodeId), state);
+                showPlaylist(songdata, originalPlaylist, &chosenRow, &(uis->chosenNodeId), state, settings);
                 state->uiState.resetPlaylistDisplay = false;
                 fflush(stdout);
         }
         else if (state->currentView == SEARCH_VIEW && refresh)
         {
                 clearScreen();
-                showSearch(songdata, &chosenSearchResultRow, ui);
+                showSearch(songdata, &chosenSearchResultRow, ui, settings);
                 refresh = false;
                 fflush(stdout);
         }
         else if (state->currentView == LIBRARY_VIEW && refresh)
         {
-                showLibrary(songdata, state);
+                showLibrary(songdata, state, settings);
                 fflush(stdout);
         }
         else if (state->currentView == TRACK_VIEW)
         {
-                showTrackView(settings, songdata, state, elapsedSeconds);
+                showTrackView(term_w, term_h, settings, songdata, state, elapsedSeconds);
                 fflush(stdout);
         }
 
