@@ -99,7 +99,7 @@ void skip(void)
                 refresh = true;
 }
 
-void updateLastSongSwitchTime(void)
+void resetStartTime(void)
 {
         clock_gettime(CLOCK_MONOTONIC, &start_time);
 }
@@ -272,7 +272,7 @@ void prepareIfSkippedSilent(void)
         {
                 skipping = true;
                 hasSilentlySwitched = false;
-                updateLastSongSwitchTime();
+                resetStartTime();
                 setCurrentImplementationType(NONE);
                 setRepeatEnabled(false);
                 audioData.endOfListReached = false;
@@ -1407,6 +1407,22 @@ Node *getSongByNumber(PlayList *playlist, int songNumber)
         return song;
 }
 
+void addToFavoritesPlaylist(void)
+{
+        if (currentSong == NULL)
+                return;
+
+        int id = currentSong->id;
+
+        Node *node = NULL;
+
+        if (findSelectedEntryById(favoritesPlaylist, id) != NULL) // Song is already in list
+                return;
+
+        createNode(&node, currentSong->song.filePath, id);
+        addToList(favoritesPlaylist, node);
+}
+
 int loadDecoder(SongData *songData, bool *songDataDeleted)
 {
         int result = 0;
@@ -1966,6 +1982,8 @@ void *updateLibraryThread(void *arg)
         char *path = (char *)arg;
         int tmpDirectoryTreeEntries = 0;
 
+        setErrorMessage("Updating Library...");
+
         FileSystemEntry *tmp = createDirectoryTree(path, &tmpDirectoryTreeEntries);
 
         if (!tmp)
