@@ -59,7 +59,7 @@ void handleRemove(void);
 
 FileSystemEntry *enqueueSongs(FileSystemEntry *entry, UIState *uis);
 
-void updateLastSongSwitchTime(void);
+void resetStartTime(void);
 
 void playbackPause(struct timespec *pause_time);
 
@@ -165,4 +165,6 @@ void markListAsEnqueued(FileSystemEntry *root, PlayList *playlist);
 
 bool isContainedWithin(FileSystemEntry *entry, FileSystemEntry *containingEntry);
 
+void addToFavoritesPlaylist(void);
+
 #endif
