@@ -62,6 +62,7 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.progressBarCurrentEvenChar, "━", sizeof(settings.progressBarCurrentEvenChar));
         c_strcpy(settings.progressBarCurrentOddChar, "━", sizeof(settings.progressBarCurrentOddChar));
         c_strcpy(settings.saveRepeatShuffleSettings, "1", sizeof(settings.saveRepeatShuffleSettings));
+        c_strcpy(settings.trackTitleAsWindowTitle, "1", sizeof(settings.trackTitleAsWindowTitle));
 #ifdef __APPLE__
         // Visualizer looks wonky in default terminal but let's enable it anyway. People need to switch
         c_strcpy(settings.visualizerEnabled, "1", sizeof(settings.visualizerEnabled));
@@ -96,7 +97,7 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.seekForward, "d", sizeof(settings.seekForward));
         c_strcpy(settings.savePlaylist, "x", sizeof(settings.savePlaylist));
         c_strcpy(settings.updateLibrary, "u", sizeof(settings.updateLibrary));
-        c_strcpy(settings.addToMainPlaylist, ".", sizeof(settings.addToMainPlaylist));
+        c_strcpy(settings.addToFavoritesPlaylist, ".", sizeof(settings.addToFavoritesPlaylist));
         c_strcpy(settings.hardPlayPause, " ", sizeof(settings.hardPlayPause));
         c_strcpy(settings.hardSwitchNumberedSong, "\n", sizeof(settings.hardSwitchNumberedSong));
         c_strcpy(settings.hardPrev, "[D", sizeof(settings.hardPrev));
@@ -128,13 +129,13 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.prevPage, "[5~", sizeof(settings.prevPage));
         c_strcpy(settings.hardRemove, "[3~", sizeof(settings.hardRemove));
         c_strcpy(settings.hardRemove2, "[P", sizeof(settings.hardRemove2));
-        c_strcpy(settings.mouseLeftClick, "[M ", sizeof(settings.mouseLeftClick));
-        c_strcpy(settings.mouseMiddleClick, "[M!", sizeof(settings.mouseMiddleClick));
-        c_strcpy(settings.mouseRightClick, "[M\"", sizeof(settings.mouseRightClick));
-        c_strcpy(settings.mouseScrollUp, "[M`", sizeof(settings.mouseScrollUp));
-        c_strcpy(settings.mouseScrollDown, "[Ma", sizeof(settings.mouseScrollDown));
-        c_strcpy(settings.mouseAltScrollUp, "[Mh", sizeof(settings.mouseAltScrollUp));
-        c_strcpy(settings.mouseAltScrollDown, "[Mi", sizeof(settings.mouseAltScrollDown));
+        c_strcpy(settings.mouseLeftClick, "[<0", sizeof(settings.mouseLeftClick));
+        c_strcpy(settings.mouseMiddleClick, "[<1", sizeof(settings.mouseMiddleClick));
+        c_strcpy(settings.mouseRightClick, "[<2", sizeof(settings.mouseRightClick));
+        c_strcpy(settings.mouseScrollUp, "[<64", sizeof(settings.mouseScrollUp));
+        c_strcpy(settings.mouseScrollDown, "[<65", sizeof(settings.mouseScrollDown));
+        c_strcpy(settings.mouseAltScrollUp, "[<72", sizeof(settings.mouseAltScrollUp));
+        c_strcpy(settings.mouseAltScrollDown, "[<73", sizeof(settings.mouseAltScrollDown));
         c_strcpy(settings.lastVolume, "100", sizeof(settings.lastVolume));
         c_strcpy(settings.color, "6", sizeof(settings.color));
         c_strcpy(settings.artistColor, "6", sizeof(settings.artistColor));
@@ -267,9 +268,9 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.savePlaylist, sizeof(settings.savePlaylist), "%s", pair->value);
                 }
-                else if (strcmp(lowercaseKey, "addtomainplaylist") == 0)
+                else if (strcmp(lowercaseKey, "addtofavoritesplaylist") == 0)
                 {
-                        snprintf(settings.quit, sizeof(settings.quit), "%s", pair->value);
+                        snprintf(settings.addToFavoritesPlaylist, sizeof(settings.addToFavoritesPlaylist), "%s", pair->value);
                 }
                 else if (strcmp(lowercaseKey, "lastvolume") == 0)
                 {
@@ -311,6 +312,10 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.saveRepeatShuffleSettings, sizeof(settings.saveRepeatShuffleSettings), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "tracktitleaswindowtitle") == 0)
+                {
+                        snprintf(settings.trackTitleAsWindowTitle, sizeof(settings.trackTitleAsWindowTitle), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "replaygaincheckfirst") == 0)
                 {
                         snprintf(settings.replayGainCheckFirst, sizeof(settings.replayGainCheckFirst), "%s", pair->value);
@@ -587,7 +592,7 @@ void mapSettingsToKeys(AppSettings *settings, UISettings *ui, EventMapping *mapp
         mappings[16] = (EventMapping){settings->toggleRepeat, EVENT_TOGGLEREPEAT};
         mappings[17] = (EventMapping){settings->savePlaylist, EVENT_EXPORTPLAYLIST};
         mappings[18] = (EventMapping){settings->toggleColorsDerivedFrom, EVENT_TOGGLEPROFILECOLORS};
-        mappings[19] = (EventMapping){settings->addToMainPlaylist, EVENT_ADDTOMAINPLAYLIST};
+        mappings[19] = (EventMapping){settings->addToFavoritesPlaylist, EVENT_ADDTOFAVORITESPLAYLIST};
         mappings[20] = (EventMapping){settings->updateLibrary, EVENT_UPDATELIBRARY};
         mappings[21] = (EventMapping){settings->hardPlayPause, EVENT_PLAY_PAUSE};
         mappings[22] = (EventMapping){settings->hardPrev, EVENT_PREV};
@@ -830,6 +835,7 @@ void getConfig(AppSettings *settings, UISettings *ui)
         ui->hideLogo = (settings->hideLogo[0] == '1');
         ui->hideHelp = (settings->hideHelp[0] == '1');
         ui->saveRepeatShuffleSettings = (settings->saveRepeatShuffleSettings[0] == '1');
+        ui->trackTitleAsWindowTitle = (settings->trackTitleAsWindowTitle[0] == '1');
 
         int tmp = getNumber(settings->color);
         if (tmp >= 0)
@@ -953,6 +959,8 @@ void setConfig(AppSettings *settings, UISettings *ui)
                 ui->hideGlimmeringText ? c_strcpy(settings->hideGlimmeringText, "1", sizeof(settings->hideGlimmeringText)) : c_strcpy(settings->hideGlimmeringText, "0", sizeof(settings->hideGlimmeringText));
         if (settings->mouseEnabled[0] == '\0')
                 ui->mouseEnabled ? c_strcpy(settings->mouseEnabled, "1", sizeof(settings->mouseEnabled)) : c_strcpy(settings->mouseEnabled, "0", sizeof(settings->mouseEnabled));
+        if (settings->trackTitleAsWindowTitle[0] == '\0')
+                ui->trackTitleAsWindowTitle ? c_strcpy(settings->trackTitleAsWindowTitle, "1", sizeof(settings->trackTitleAsWindowTitle)) : c_strcpy(settings->trackTitleAsWindowTitle, "0", sizeof(settings->trackTitleAsWindowTitle));
 
         snprintf(settings->repeatState, sizeof(settings->repeatState), "%d", ui->repeatState);
 
@@ -1041,6 +1049,9 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "repeatState=%s\n\n", settings->repeatState);
         fprintf(file, "shuffleEnabled=%s\n\n", settings->shuffleEnabled);
 
+        fprintf(file, "# Set the window title to the title of the currently playing track\n");
+        fprintf(file, "trackTitleAsWindowTitle=%s\n\n", settings->trackTitleAsWindowTitle);
+
         fprintf(file, "\n[visualizer]\n\n");
         fprintf(file, "visualizerEnabled=%s\n", settings->visualizerEnabled);
         fprintf(file, "visualizerHeight=%s\n", settings->visualizerHeight);
@@ -1150,7 +1161,7 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "seekBackward=%s\n", settings->seekBackward);
         fprintf(file, "seekForward=%s\n", settings->seekForward);
         fprintf(file, "savePlaylist=%s\n", settings->savePlaylist);
-        fprintf(file, "addToMainPlaylist=%s\n", settings->addToMainPlaylist);
+        fprintf(file, "addToFavoritesPlaylist=%s\n", settings->addToFavoritesPlaylist);
         fprintf(file, "updateLibrary=%s\n", settings->updateLibrary);
         fprintf(file, "moveSongUp=%s\n", settings->moveSongUp);
         fprintf(file, "moveSongDown=%s\n", settings->moveSongDown);
