@@ -9,6 +9,7 @@
 #include <cstring>
 #include <fstream>
 #include <iostream>
+#include <unordered_map>
 
 #include <taglib/attachedpictureframe.h>
 #include <taglib/fileref.h>
@@ -20,8 +21,13 @@
 #include <taglib/mp4coverart.h>
 #include <taglib/vorbisfile.h>
 #include <taglib/opusfile.h>
+#include <taglib/wavfile.h>
 #include <taglib/xiphcomment.h>
 #include <taglib/tag.h>
+#include <taglib/apefooter.h>
+#include <taglib/apeitem.h>
+#include <taglib/apetag.h>
+#include <taglib/textidentificationframe.h>
 #include <ogg/ogg.h>
 
 /*
@@ -50,6 +56,12 @@ static inline bool is_base64(unsigned char c)
         return (isalnum(c) || (c == '+') || (c == '/'));
 }
 
+#if defined(TAGLIB_MAJOR_VERSION) && TAGLIB_MAJOR_VERSION >= 2
+#define HAVE_COMPLEXPROPERTIES 1
+#else
+#define HAVE_COMPLEXPROPERTIES 0
+#endif
+
 // Function to decode Base64-encoded data
 std::vector<unsigned char> decodeBase64(const std::string &encoded_string)
 {
@@ -98,6 +110,34 @@ std::vector<unsigned char> decodeBase64(const std::string &encoded_string)
         return decoded_data;
 }
 
+TagLib::StringList getOggFieldListCaseInsensitive(const TagLib::Ogg::XiphComment *comment,
+                                                  const std::string &fieldName)
+{
+        // Create a mutable copy of fieldName for transformation
+        std::string lowerFieldName = fieldName;
+        std::transform(lowerFieldName.begin(), lowerFieldName.end(), lowerFieldName.begin(),
+                       [](unsigned char c)
+                       { return std::tolower(c); });
+
+        TagLib::Ogg::FieldListMap fieldListMap = comment->fieldListMap();
+
+        for (auto it = fieldListMap.begin(); it != fieldListMap.end(); ++it)
+        {
+                // Create a mutable copy of current key for transformation
+                std::string currentKey(it->first.toCString());
+                std::transform(currentKey.begin(), currentKey.end(), currentKey.begin(),
+                               [](unsigned char c)
+                               { return std::tolower(c); });
+
+                if (currentKey == lowerFieldName)
+                {
+                        return it->second;
+                }
+        }
+
+        return TagLib::StringList();
+}
+
 extern "C"
 {
         // Function to read a 32-bit unsigned integer from buffer in big-endian format
@@ -142,7 +182,53 @@ extern "C"
                 imageData.assign(&ptr[offset], &ptr[offset + dataLength]);
         }
 
-        int extractCoverArtFromOgg(const std::string &audioFilePath, const std::string &outputFileName)
+#if HAVE_COMPLEXPROPERTIES
+
+        bool extractCoverArtFromOgg(const std::string &audioFilePath, const std::string &outputFileName)
+        {
+                TagLib::FileRef f(audioFilePath.c_str(), true);
+                if (!f.file() || !f.file()->isOpen())
+                {
+                        std::cerr << "Error: Could not open file: " << audioFilePath << std::endl;
+                        return false;
+                }
+
+                auto pictures = f.file()->complexProperties("PICTURE");
+                if (pictures.isEmpty())
+                {
+                        std::cerr << "No cover art found in the file (via complexProperties).\n";
+                        return false;
+                }
+
+                // Use the first picture found (usually the front cover)
+                for (const auto &pic : pictures)
+                {
+                        auto it_data = pic.find("data");
+
+                        if (it_data == pic.end())
+                                continue; // Skip if no data
+
+                        // Write the image data to a file
+                        std::ofstream outFile(outputFileName, std::ios::binary);
+                        if (!outFile)
+                        {
+                                std::cerr << "Error: Could not write to output file.\n";
+                                return false;
+                        }
+                        TagLib::ByteVector bv = it_data->second.toByteVector();
+                        outFile.write(bv.data(), bv.size());
+                        outFile.close();
+
+                        return true;
+                }
+
+                std::cerr << "No usable cover image found in complexProperties().\n";
+                return false;
+        }
+
+#else
+
+        bool extractCoverArtFromOgg(const std::string &audioFilePath, const std::string &outputFileName)
         {
                 TagLib::File *file = nullptr;
                 TagLib::Tag *tag = nullptr;
@@ -173,7 +259,9 @@ extern "C"
                 }
 
                 // Check METADATA_BLOCK_PICTURE
-                TagLib::StringList pictureList = xiphComment->fieldListMap()["METADATA_BLOCK_PICTURE"];
+                TagLib::StringList pictureList = getOggFieldListCaseInsensitive(xiphComment,
+                                                                                "METADATA_BLOCK_PICTURE");
+
                 if (!pictureList.isEmpty())
                 {
                         std::string base64Data = pictureList.front().to8Bit(true);
@@ -191,14 +279,17 @@ extern "C"
                                 return false; // Could not write to output file
                         }
                         outFile.write(reinterpret_cast<const char *>(imageData.data()), imageData.size());
+
                         outFile.close();
                         delete file;
-                        return 0; // Success
+                        return true; // Success
                 }
 
                 // Check COVERART and COVERARTMIME
-                TagLib::StringList coverArtList = xiphComment->fieldListMap()["COVERART"];
-                TagLib::StringList coverArtMimeList = xiphComment->fieldListMap()["COVERARTMIME"];
+                TagLib::StringList coverArtList = getOggFieldListCaseInsensitive(xiphComment,
+                                                                                 "COVERART");
+                TagLib::StringList coverArtMimeList = getOggFieldListCaseInsensitive(xiphComment,
+                                                                                     "COVERARTMIME");
                 if (!coverArtList.isEmpty() && !coverArtMimeList.isEmpty())
                 {
                         std::string base64Data = coverArtList.front().to8Bit(true);
@@ -222,13 +313,57 @@ extern "C"
                 return false; // No cover art found
         }
 
+#endif
+
+        bool looksLikeJpeg(const std::vector<unsigned char> &data)
+        {
+                return data.size() > 4 &&
+                       data[0] == 0xFF && data[1] == 0xD8 &&
+                       data[data.size() - 2] == 0xFF && data[data.size() - 1] == 0xD9;
+        }
+        bool looksLikePng(const std::vector<unsigned char> &data)
+        {
+                static const unsigned char pngHeader[8] = {0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A};
+                if (data.size() < 12)
+                        return false;
+                if (memcmp(data.data(), pngHeader, 8) != 0)
+                        return false;
+                // Look for IEND within last 16 bytes
+                for (size_t i = data.size() >= 16 ? data.size() - 16 : 0; i + 3 < data.size(); ++i)
+                {
+                        if (memcmp(&data[i], "IEND", 4) == 0)
+                                return true;
+                }
+                return false;
+        }
+        bool looksLikeWebp(const std::vector<unsigned char> &data)
+        {
+                return data.size() > 12 &&
+                       memcmp(&data[0], "RIFF", 4) == 0 &&
+                       memcmp(&data[8], "WEBP", 4) == 0;
+        }
+        bool isAudioOggStreamHeader(const ogg_packet &headerPacket)
+        {
+                if (headerPacket.bytes >= 7 &&
+                    headerPacket.packet[0] == 0x01 &&
+                    memcmp(headerPacket.packet + 1, "vorbis", 6) == 0)
+                        return true;
+                if (headerPacket.bytes >= 8 &&
+                    memcmp(headerPacket.packet, "OpusHead", 8) == 0)
+                        return true;
+                if (headerPacket.bytes >= 4 &&
+                    memcmp(headerPacket.packet, "fLaC", 4) == 0)
+                        return true;
+                return false;
+        }
+
         bool extractCoverArtFromOggVideo(const std::string &audioFilePath, const std::string &outputFileName)
         {
                 FILE *oggFile = fopen(audioFilePath.c_str(), "rb");
                 if (!oggFile)
                 {
-                        std::cerr << "Error: Could not open file." << std::endl;
-                        return false; // File not found or could not be opened
+                        std::cerr << "Error: Could not open file: " << audioFilePath << std::endl;
+                        return false;
                 }
 
                 ogg_sync_state oy;
@@ -238,14 +373,13 @@ extern "C"
                 ogg_packet op;
 
                 std::map<int, ogg_stream_state> streams;
-                std::map<int, std::vector<unsigned char>> imageDataStreams;
-                std::map<int, bool> isImageStream; // Track if a stream is an image (JPEG/PNG)
+                std::map<int, bool> isHeaderParsed;
+                std::map<int, bool> isAudioStream;
+                std::map<int, std::vector<unsigned char>> streamPackets;
 
                 char *buffer;
-                int bytes;
-                bool coverArtFound = false;
+                int bytes = 0;
 
-                // Read through the Ogg container to find and extract PNG or JPEG streams
                 while (true)
                 {
                         buffer = ogg_sync_buffer(&oy, 4096);
@@ -256,94 +390,68 @@ extern "C"
                         {
                                 int serialNo = ogg_page_serialno(&og);
 
-                                // Initialize stream if not already done
+                                // Lazily create state for new streams
                                 if (streams.find(serialNo) == streams.end())
                                 {
                                         ogg_stream_state os;
                                         ogg_stream_init(&os, serialNo);
                                         streams[serialNo] = os;
-                                        isImageStream[serialNo] = false; // Start by assuming this is not an image stream
+                                        isHeaderParsed[serialNo] = false;
+                                        isAudioStream[serialNo] = false;
                                 }
-
                                 ogg_stream_state &os = streams[serialNo];
                                 ogg_stream_pagein(&os, &og);
 
                                 while (ogg_stream_packetout(&os, &op) == 1)
                                 {
-                                        // Check the packet for PNG signature (\x89PNG\r\n\x1A\n) or JPEG SOI (\xFF\xD8)
-                                        if (op.bytes >= 8 && std::memcmp(op.packet, "\x89PNG\r\n\x1A\n", 8) == 0)
+                                        // Only decide on audio-ness for first packet
+                                        if (!isHeaderParsed[serialNo])
                                         {
-                                                isImageStream[serialNo] = true;
-                                        }
-                                        else if (op.bytes >= 2 && op.packet[0] == 0xFF && op.packet[1] == 0xD8)
-                                        {
-                                                isImageStream[serialNo] = true;
-                                        }
-                                        else if (op.bytes >= 12 && std::memcmp(op.packet, "RIFF", 4) == 0 && std::memcmp(op.packet + 8, "WEBP", 4) == 0)
-                                        {
-                                                isImageStream[serialNo] = true;
-                                        }
+                                                isAudioStream[serialNo] = isAudioOggStreamHeader(op);
+                                                isHeaderParsed[serialNo] = true;
 
-                                        // If this is a detected image stream (either PNG or JPEG), collect the packet data
-                                        if (isImageStream[serialNo])
+                                                if (isAudioStream[serialNo])
+                                                        continue;
+                                        }
+                                        if (!isAudioStream[serialNo])
                                         {
-                                                imageDataStreams[serialNo].insert(
-                                                    imageDataStreams[serialNo].end(),
+                                                // For image/video: collect all packets from this stream
+                                                streamPackets[serialNo].insert(
+                                                    streamPackets[serialNo].end(),
                                                     op.packet,
                                                     op.packet + op.bytes);
                                         }
                                 }
                         }
-
-                        // Stop reading if the file ends
                         if (bytes == 0)
-                        {
                                 break;
-                        }
                 }
 
-                fclose(oggFile);
+                // Clean up
+                for (auto &entry : streams)
+                        ogg_stream_clear(&(entry.second));
                 ogg_sync_clear(&oy);
+                fclose(oggFile);
 
-                // Process collected image data streams
-                for (const auto &entry : imageDataStreams)
+                // Write out a valid image stream
+                for (auto &kv : streamPackets)
                 {
-                        if (isImageStream[entry.first] && !entry.second.empty())
+                        const auto &data = kv.second;
+                        if (looksLikeJpeg(data) || looksLikePng(data) || looksLikeWebp(data))
                         {
-                                // Save the image data to a file using outputFileName
                                 std::ofstream outFile(outputFileName, std::ios::binary);
                                 if (!outFile)
                                 {
                                         std::cerr << "Error: Could not write to output file." << std::endl;
-                                        // Clean up stream states
-                                        for (auto &streamEntry : streams)
-                                        {
-                                                ogg_stream_clear(&streamEntry.second);
-                                        }
-                                        return false; // Could not write to output file
+                                        return false;
                                 }
-                                outFile.write(reinterpret_cast<const char *>(entry.second.data()), entry.second.size());
+                                outFile.write(reinterpret_cast<const char *>(data.data()), data.size());
                                 outFile.close();
-
-                                coverArtFound = true;
-                                break; // Stop after finding and writing the first cover art
+                                return true;
                         }
                 }
-
-                // Clean up stream states
-                for (auto &streamEntry : streams)
-                {
-                        ogg_stream_clear(&streamEntry.second);
-                }
-
-                // Return whether the cover art was successfully found and written
-                if (!coverArtFound)
-                {
-                        std::cerr << "No cover art found in the file." << std::endl;
-                        return false; // No cover art found
-                }
-
-                return true; // Success
+                std::cerr << "No embedded image stream found in file." << std::endl;
+                return false;
         }
 
         bool extractCoverArtFromMp3(const std::string &inputFile, const std::string &coverFilePath)
@@ -434,6 +542,68 @@ extern "C"
                 return false;
         }
 
+        bool extractCoverArtFromWav(const std::string &inputFile, const std::string &coverFilePath)
+        {
+                TagLib::RIFF::WAV::File file(inputFile.c_str());
+                if (!file.isValid())
+                {
+                        return false;
+                }
+
+                const TagLib::ID3v2::Tag *id3v2tag = file.ID3v2Tag();
+                if (id3v2tag)
+                {
+                        // Collect all attached picture frames
+                        TagLib::ID3v2::FrameList frames;
+                        frames.append(id3v2tag->frameListMap()["APIC"]);
+                        frames.append(id3v2tag->frameListMap()["PIC"]);
+
+                        if (!frames.isEmpty())
+                        {
+                                for (auto it = frames.begin(); it != frames.end(); ++it)
+                                {
+                                        const TagLib::ID3v2::AttachedPictureFrame *picFrame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(*it);
+                                        if (picFrame)
+                                        {
+                                                // Access picture data and MIME type
+                                                TagLib::ByteVector pictureData = picFrame->picture();
+                                                TagLib::String mimeType = picFrame->mimeType();
+
+                                                // Construct the output file path
+                                                std::string outputFilePath = coverFilePath;
+
+                                                // Write the image data to a file
+                                                FILE *outFile = fopen(outputFilePath.c_str(), "wb");
+                                                if (outFile)
+                                                {
+                                                        fwrite(pictureData.data(), 1, pictureData.size(), outFile);
+                                                        fclose(outFile);
+
+                                                        return true;
+                                                }
+                                                else
+                                                {
+                                                        return false; // Failed to open output file
+                                                }
+
+                                                // Break if only the first image is needed
+                                                break;
+                                        }
+                                }
+                        }
+                        else
+                        {
+                                return false; // No picture frames found
+                        }
+                }
+                else
+                {
+                        return false; // No ID3v2 tag found
+                }
+
+                return true; // Success
+        }
+
         bool extractCoverArtFromOpus(const std::string &audioFilePath, const std::string &outputFileName)
         {
                 int error;
@@ -457,7 +627,7 @@ extern "C"
                 {
                         // Check for METADATA_BLOCK_PICTURE
                         const char *comment = tags->user_comments[i];
-                        if (strncmp(comment, "METADATA_BLOCK_PICTURE=", 23) == 0)
+                        if (strncasecmp(comment, "METADATA_BLOCK_PICTURE=", 23) == 0)
                         {
                                 // Extract the value after "METADATA_BLOCK_PICTURE="
                                 std::string metadataBlockPicture(comment + 23);
@@ -567,7 +737,8 @@ extern "C"
         {
                 TagLib::MP4::File file(inputFile.c_str());
 
-                if (!file.isValid()) {
+                if (!file.isValid())
+                {
                         return false;
                 }
 
@@ -634,7 +805,7 @@ extern "C"
 
                         // Copy the result into the output char* title, ensuring no overflow
                         c_strcpy(title, extractedTitle.c_str(), titleMaxLength - 1); // Copy up to titleMaxLength - 1 characters
-                        title[titleMaxLength - 1] = '\0';                           // Null-terminate the string
+                        title[titleMaxLength - 1] = '\0';                            // Null-terminate the string
                 }
                 else
                 {
@@ -643,10 +814,35 @@ extern "C"
                 }
         }
 
+        double parseDecibelValue(const TagLib::String &dbString)
+        {
+                double val = 0.0;
+                try
+                {
+                        std::string valStr = dbString.to8Bit(true);
+                        std::string filtered;
+                        for (char c : valStr)
+                        {
+                                if (std::isdigit((unsigned char)c) || c == '.' || c == '-' || c == '+' || c == 'e' || c == 'E')
+                                {
+                                        filtered.push_back(c);
+                                }
+                        }
+                        val = std::stod(filtered);
+                }
+                catch (...)
+                {
+                }
+                return val;
+        }
+
         int extractTags(const char *input_file, TagSettings *tag_settings, double *duration, const char *coverFilePath)
         {
                 memset(tag_settings, 0, sizeof(TagSettings)); // Initialize tag settings
 
+                tag_settings->replaygainTrack = 0.0;
+                tag_settings->replaygainAlbum = 0.0;
+
                 // Use TagLib's FileRef for generic file parsing.
                 TagLib::FileRef f(input_file);
                 if (f.isNull() || !f.file())
@@ -693,6 +889,11 @@ extern "C"
 
                         // Copy the year as date
                         snprintf(tag_settings->date, sizeof(tag_settings->date), "%d", (int)tag->year());
+
+                        if (tag_settings->date[0] == '0')
+                        {
+                                tag_settings->date[0] = '\0';
+                        }
                 }
 
                 // Extract audio properties for duration.
@@ -707,7 +908,95 @@ extern "C"
                         return -2;
                 }
 
-                // Cover art extraction using format-specific classes if necessary.
+                // Extract replay gain information
+                if (std::string(input_file).find(".mp3") != std::string::npos)
+                {
+                        TagLib::MPEG::File mp3File(input_file);
+                        TagLib::ID3v2::Tag *id3v2Tag = mp3File.ID3v2Tag();
+
+                        if (id3v2Tag)
+                        {
+                                // Retrieve all TXXX frames
+                                TagLib::ID3v2::FrameList frames = id3v2Tag->frameList("TXXX");
+                                for (TagLib::ID3v2::FrameList::Iterator it = frames.begin();
+                                     it != frames.end(); ++it)
+                                {
+                                        // Cast to the user-text (TXXX) frame class
+                                        TagLib::ID3v2::TextIdentificationFrame *txxx =
+                                            dynamic_cast<TagLib::ID3v2::TextIdentificationFrame *>(*it);
+                                        if (!txxx)
+                                                continue;
+
+                                        TagLib::StringList fields = txxx->fieldList();
+                                        if (fields.size() >= 2)
+                                        {
+                                                TagLib::String desc = fields[0];
+                                                TagLib::String val = fields[1];
+
+                                                if (desc.upper() == "REPLAYGAIN_TRACK_GAIN")
+                                                {
+                                                        tag_settings->replaygainTrack = parseDecibelValue(val);
+                                                }
+                                                else if (desc.upper() == "REPLAYGAIN_ALBUM_GAIN")
+                                                {
+                                                        tag_settings->replaygainAlbum = parseDecibelValue(val);
+                                                }
+                                        }
+                                }
+                        }
+
+                        TagLib::APE::Tag *apeTag = mp3File.APETag();
+
+                        if (apeTag)
+                        {
+                                TagLib::APE::ItemListMap items = apeTag->itemListMap();
+                                for (auto it = items.begin(); it != items.end(); ++it)
+                                {
+                                        std::string key = it->first.upper().toCString();
+                                        TagLib::String value = it->second.toString();
+
+                                        if (key == "REPLAYGAIN_TRACK_GAIN")
+                                        {
+                                                tag_settings->replaygainTrack = parseDecibelValue(value);
+                                        }
+                                        else if (key == "REPLAYGAIN_ALBUM_GAIN")
+                                        {
+                                                tag_settings->replaygainAlbum = parseDecibelValue(value);
+                                        }
+                                }
+                        }
+                }
+                else if (std::string(input_file).find(".flac") != std::string::npos)
+                {
+                        TagLib::FLAC::File flacFile(input_file);
+                        TagLib::Ogg::XiphComment *xiphComment = flacFile.xiphComment();
+
+                        if (xiphComment)
+                        {
+                                const TagLib::Ogg::FieldListMap &fieldMap = xiphComment->fieldListMap();
+
+                                auto trackGainIt = fieldMap.find("REPLAYGAIN_TRACK_GAIN");
+                                if (trackGainIt != fieldMap.end())
+                                {
+                                        const TagLib::StringList &trackGainList = trackGainIt->second;
+                                        if (!trackGainList.isEmpty())
+                                        {
+                                                tag_settings->replaygainTrack = parseDecibelValue(trackGainList.front());
+                                        }
+                                }
+
+                                auto albumGainIt = fieldMap.find("REPLAYGAIN_ALBUM_GAIN");
+                                if (albumGainIt != fieldMap.end())
+                                {
+                                        const TagLib::StringList &albumGainList = albumGainIt->second;
+                                        if (!albumGainList.isEmpty())
+                                        {
+                                                tag_settings->replaygainAlbum = parseDecibelValue(albumGainList.front());
+                                        }
+                                }
+                        }
+                }
+
                 std::string filename(input_file);
                 std::string extension = filename.substr(filename.find_last_of('.') + 1);
                 bool coverArtExtracted = false;
@@ -730,13 +1019,17 @@ extern "C"
                 }
                 else if (extension == "ogg")
                 {
-                        coverArtExtracted = extractCoverArtFromOggVideo(input_file, coverFilePath);
+                        coverArtExtracted = extractCoverArtFromOgg(input_file, coverFilePath);
 
                         if (!coverArtExtracted)
                         {
-                                coverArtExtracted = extractCoverArtFromOgg(input_file, coverFilePath);
+                                coverArtExtracted = extractCoverArtFromOggVideo(input_file, coverFilePath);
                         }
                 }
+                else if (extension == "wav")
+                {
+                        coverArtExtracted = extractCoverArtFromWav(input_file, coverFilePath);
+                }
 
                 if (coverArtExtracted)
                 {
